%%% Replication of sumulation evidence on algorithm convergence for Table 2
%%% Create data csv files with the columns:
%%% Num. of iterations || -1/log(\nu_max) || log(\lam_min) || |gamma|_max
%%% -------------------------------------------------------------------

clear; clc; close all;

% Set of dimensions for correlation matrices
dim_list = [5;10;25]; J = length(dim_list);

% Cell array preallocation for the results
M = 50000;   % number of simulated correlation matrices
sim_data = cell(J,1);
for j = 1:J
    sim_data{j} = zeros(M,4);
end

% Generate dandom scaling coefficients 
% (to get a sufficient variation of \nu_max and \lam_min)
h = rand(M,1)*[5,4,3];

% Simulations
for j = 1:J
    n = dim_list(j);
    ng = n*(n-1)/2;
 
    for m = 1:M
        gamma = h(m,j)*(2*rand(ng,1)-1);
        gamma_abs_max = max(abs(gamma));  
        [C,logC,iter_number] = gamma_inverse_mapping(gamma,1e-8);
        lamC_min = min(eig(C));

        [~,~,J_diag] = derivative_expm(logC);
        lamJ_max = max(eig(J_diag));
        
        sim_data{j}(m,1) = iter_number;
        if isreal(lamJ_max)
            sim_data{j}(m,2) = -1/log(lamJ_max);
        else
            sim_data{j}(m,2) = NaN;
        end
        if isreal(lamC_min)
            sim_data{j}(m,3) = log(lamC_min);
        else
            sim_data{j}(m,3) = NaN;
        end
        sim_data{j}(m,4) = gamma_abs_max;
        
        if mod(m,10000) == 0
            fprintf(['n = ' num2str(n) ': ' num2str(m) ' out of ' num2str(M) ' \n']);
        end
    end

    % Store results
    csvwrite(['iter_simul_data_n',num2str(n),'.csv'],sim_data{j})
end