%%% New transformation for correlation matrices [Inverse mapping]
%%% -------------------------------------------------------------------

function [C,G,iter_number] = gamma_inverse_mapping(gamma,tol_value)
    
    C = []; G = [];
    iter_number = -1;
    
    % Check input
    n = 0.5*(1+sqrt(1+8*length(gamma)));
    if isvector(gamma) && (rem(n,1)==0)... 
        && (tol_value >= 1e-15 && tol_value <= 1e-4)
       
        % Place elements from gamma into off-diagonal parts 
        % and put zeros on the main diagonal of nxn symmetric matrix A
        G = zeros(n);
        G(logical(tril(ones(n),-1))) = gamma;
        G = G + G';
        
        % Read some properties of matrix A
        diag_vec = diag(G);
        diag_ind = logical(eye(n));
      
        % Iterative algorithm to get the proper diagonal vector
        conv = max(sqrt(n),sqrt(n)*tol_value);
        while conv > sqrt(n)*tol_value
            diag_delta = log(diag(expm(G)));
            diag_vec = diag_vec - diag_delta;
            G(diag_ind) = diag_vec;
            conv = norm(diag_delta);
            iter_number = iter_number + 1;
        end
        
        % Get a unique reciprocal correlation matrix
        C = expm(G);
        %C(diag_ind) = ones(n,1);

    else
        fprintf('Error : input is of wrong format');
    end